package writer;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import json.GrammarToJSON;
import word_generator.WordGenerator;

public class Writer {

	private static final int DISTINCT_WORDS = 5;
	private static final int MAX_LENGTH = 5;

	GrammarToJSON jg = new GrammarToJSON();
	WordGenerator wg = new WordGenerator();

	private List<String> createGrammarWordPairs(String[] vars, int alphaSize, int maxConclusios, int maxConclusioSize,
			String start, int type, int mode, int n) {
		List<String> grammarWordPairs = new LinkedList<>();
		List<String> jsonGrammars = jg.createJSONGrammars(vars, alphaSize, maxConclusios, maxConclusioSize, start, type,
				mode, n);
		for (String jsonGrammar : jsonGrammars) {
			Set<String> distinctWords = new HashSet<>();
			int i = 0;
			while (i < DISTINCT_WORDS) {
				String word = wg.wordGenerator(MAX_LENGTH, alphaSize);
				if (!distinctWords.contains(word)) {
					distinctWords.add(word);
					String grammarWordPair = createGrammarWordPair(jsonGrammar, word);
					grammarWordPairs.add(grammarWordPair);
					i++;
				}
			}
		}
		return grammarWordPairs;
	}

	private String createGrammarWordPair(String jsonGrammar, String word) {
		StringBuilder sb = new StringBuilder();
		sb.append("\"grammar\":");
		sb.append(jsonGrammar);
		sb.append(",\"word\":");
		sb.append("\"" + word + "\"");
		return sb.toString();
	}

	private void writeToFile(List<String> jsons) {
		try (BufferedWriter bw = new BufferedWriter(new FileWriter("src/writer/json.tmp"))) {
			int i = 1;
			for (String json : jsons) {
				bw.write(json);
				bw.newLine();
				bw.newLine();
				if (i != 0 && i % DISTINCT_WORDS == 0) {
					bw.newLine();
				}
				i++;
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void writeToFile(String[] vars, int alphaSize, int maxConclusios, int maxConclusioSize, String start,
			int type, int mode, int n) {
		List<String> grammarWordPairs = createGrammarWordPairs(vars, alphaSize, maxConclusios, maxConclusioSize, start,
				type, mode, n);
		writeToFile(grammarWordPairs);
	}

	public static void main(String[] args) {
		Writer w = new Writer();
		String[] vars = { "S", "A", "B" };
		int alphaSize = 2;
		int maxConclusios = 2;
		int maxConclusioSize = -1;
		String start = "S";
		int type = 3;
		int mode = 3;
		int n = 3;
		w.writeToFile(vars, alphaSize, maxConclusios, maxConclusioSize, start, type, mode, n);
	}

}
